/*
 *  This program is free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation; either version 2 of the License, or
 *  (at your option) any later version.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with this program; if not, write to the Free Software
 *  Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
 */

#include "battery.h"

bool ACPI_canSupport( )
{
	if( access( ACPI_ACCESS_TEST, R_OK ) == SUCCESS )
		return true;
	return false;
}



bool ACPI_init( )
{
	DIR * battdir;
	struct dirent *batt;
	char * name;
	char   path[50];
	battery batPtr;
	
	if ( ACPI_canSupport() == false )
	{
		fprintf( stderr, "Cannot support ACPI (the file %s don't exists)\n", ACPI_ACCESS_TEST );
		return ERROR;
	}
	
	powerState.type = SUPPORT_ACPI;
	powerState.updater = ACPI_Update; 
	
	/* counting number of batteries */
	powerState.nBatt = 0;
	powerState.batteryVector = NULL;
	
	if( (battdir = opendir ("/proc/acpi/battery")) == NULL )
	{
		fprintf( stderr,  "Cannot open /proc/acpi/battery: %s\n", strerror(errno) );
		return ERROR;
	}
	
	/* a cicle for creating as many battery_str structures 
	 * as ara the batteries on the laptop; everyone will be
	 * filled with static informations */
	while ((batt = readdir (battdir)))
	{
		
		name = batt->d_name;
		/* jump and .. */
		if (!strncmp (".", name, 1) || !strncmp ("..", name, 2))
			continue;
		
		/* adding a battery to powerState vector */
		powerState.nBatt += 1;
		
		powerState.batteryVector = realloc( powerState.batteryVector, \
		                         sizeof( battery ) * powerState.nBatt );
		
		/* malloc testing.. */
		if ( powerState.batteryVector == NULL )
		{
			powerState.nBatt -= 1;
			fprintf( stderr,  "Error calling 'realloc()': %s\n", strerror(errno));
			return ERROR;
		}
		
		powerState.batteryVector[powerState.nBatt-1] = malloc( sizeof( struct battery_str));
		
		/* once more again */
		if ( powerState.batteryVector[ powerState.nBatt-1 ] == NULL )
		{
			powerState.nBatt -= 1;
			powerState.batteryVector = realloc( powerState.batteryVector, \
			                             sizeof( battery ) * powerState.nBatt );
			fprintf( stderr, "Error calling 'malloc()': %s\n", strerror(errno));
			return ERROR;
		}
		
		batPtr = powerState.batteryVector[ powerState.nBatt - 1 ];
		
		/* we can find 'status' or even 'state' */
		snprintf( path, 50, "/proc/acpi/battery/%s/status", name );
		if (access(path, R_OK) == SUCCESS )
			snprintf( batPtr -> stateFile, 50, "/proc/acpi/battery/%s/status", name );
		else
			snprintf( batPtr -> stateFile, 50, "/proc/acpi/battery/%s/state", name );
		
		
		/* calculating maximum capacity of this battery
		 * reading file infos */
		snprintf( path, 50, "/proc/acpi/battery/%s/info", name );
		if( ACPI_maxCapacity ( path , &batPtr -> capacity ) == ERROR )
			return ERROR;		
		
		
		/* filler selection :P */
		batPtr -> filler = ACPI_Filler;
	}
	
	closedir (battdir);
	return SUCCESS;
}



bool ACPI_maxCapacity ( char * path, u_int32 * capacity )
{
	FILE *  fp;
	char    buf[512];
	char *  ptr;
	u_int16 saltCounter = 0;
	
	if( (fp = fopen( path, "r" )) == NULL ) {
		fprintf( stderr, "Error reading the info file (%s):\n%s\n", path, strerror(errno) );
		return ERROR;
	}
	
	fread_unlocked (buf, 512, 1, fp);
	fclose(fp);
	
	ptr = buf;
	while( strncmp( ptr, "design capacity:", 16 ) && saltCounter < 498)
	{
		saltCounter++;
		ptr++;
	}
	
	/* naah.. */
	if( saltCounter >= 498 )
	{
		fprintf( stderr, "The info file (%s) has not 'design capacity' tag", path );
		return ERROR;
	}
	
	ptr += 17;
	
	sscanf( ptr, "%u", capacity );
	
	return SUCCESS;
}



/* return value set to 'bool', but only for compatibility
 * with APM_Filler; its return value is never read or used */
bool ACPI_Filler ( battery bat )
{
	char *  path = bat -> stateFile;
	FILE *  fp;
	char    buf[512];
	char *  ptr;
	u_int16 saltCounter = 0;
	
	char    presentString[25];
	char    yesNoString[25];
	
	/* it prints message in case of error and then quit.. */
	if( (fp = fopen( path, "r" )) == NULL ) {
		fprintf( stderr, "Error reading the info file (%s):\n%s\n", path, strerror(errno) );
		exit( ERROR );
	}
	
	fread_unlocked (buf, 512, 1, fp);
	fclose(fp);
	
	/* in the first line there's an info that tells if the battery is present*/
	sscanf( buf, "%s %s", presentString, yesNoString);
	
	/* if not present, set actualState to zero*/
	if( !strcmp( presentString, "present:" ) && !strcmp( yesNoString, "no" ) )
	{
		bat -> actualState = 0;
	}
	else
	{
		ptr = buf;
		while( strncmp( ptr, "remaining capacity:", 19 ) && saltCounter < 495)
		{
			saltCounter++;
			ptr++;
		}
		
		/* naah.. */
		if( saltCounter >= 498 )
		{
			fprintf( stderr, "The info file (%s) has not 'remaining capacity' tag", path );
			exit( ERROR );
		}
		
		ptr += 20;
		
		sscanf( ptr, "%u", &bat -> actualState );
	}
	
	bat -> percentage = (bat -> actualState * 100) / bat -> capacity;
	
	
	return SUCCESS;
}


void ACPI_Update ( )
{
	FILE * fp;
	char *where = NULL;
	char  buf[512];
	
	register int i;
	register u_int16 percentageSum = 0;
	
	for( i = 0; i < powerState.nBatt; i++ )
	{
		powerState.batteryVector[i] -> filler( powerState.batteryVector[i] );
		percentageSum += powerState.batteryVector[i] -> percentage;
	}
	
	powerState.percentage = percentageSum / i;
	
	/* battery charger information reading
	 * thanks to wmpower */
	
	if (!(fp = fopen ("/proc/acpi/ac_adapter/0/status", "r")))
		if (!(fp = fopen ("/proc/acpi/ac_adapter/ACAD/state", "r")))
			if (!(fp = fopen ("/proc/acpi/ac_adapter/AC/state", "r")))
				if (!(fp = fopen ("/proc/acpi/ac_adapter/ADP1/state", "r")))
					return;
	
	fread_unlocked (buf, 512, 1, fp);
	fclose(fp);
	
	if (strncmp(buf, "state:",  6) == 0)
		where = buf + 26;
	if (strncmp(buf, "Status:", 7) == 0)
		where = buf + 26;
	
	if (where)
	{
		if (where[0] == 'n')
			powerState.isCharging = true;
		if (where[0] == 'f')
			powerState.isCharging = false;
	}
	
	setNewBatteryState();
	
	return;
}


bool APM_canSupport( )
{
	if( access( APM_ACCESS_TEST, R_OK ) == SUCCESS )
		return true;
	return false;
}



bool APM_init ( )
{
	
	if ( APM_canSupport() == false )
	{
		fprintf( stderr, "Cannot support APM (the file %s don't exists)\n", APM_ACCESS_TEST );
		return ERROR;
	}
	
	/* using APM, creating structure for only one battery */
	powerState.type = SUPPORT_APM;
	powerState.updater = APM_Update; 
	powerState.nBatt = 1;
	
	if( (powerState.batteryVector = malloc( sizeof( battery ) )) == NULL )
	{
		fprintf( stderr, "Error calling 'malloc()': %s\n", strerror(errno));
		return ERROR;
	}
	
	if( (powerState.batteryVector[0] = malloc( sizeof( struct battery_str ))) == NULL )
	{
		free( powerState.batteryVector );
		powerState.nBatt = 0;
		fprintf( stderr, "Error calling 'malloc()': %s\n", strerror(errno));
		return ERROR;
	}
	
	/* for APM, the only valid and updated values for a 'battery' structure
	 * are filler pointer and percent field */
	powerState.batteryVector[0] -> filler = APM_Filler;
	
	return SUCCESS;	
}


/* it returns true if battery charger is plugged in */
bool APM_Filler ( battery bat )
{
	FILE * apmFp;
	char   buf[100];
	
	char   useLess[10];
	int    useLessToo;
	
	int    recharging;
	int    percentage;
	
	/* as in case of ACPI, an error in filler make the program exit  */ 
	if( (apmFp = fopen( "/proc/apm", "r" )) == NULL )
	{
		fprintf( stderr, "Cannot open '/proc/apm' to read battery's information: %s\n", strerror(errno));
		exit( ERROR );
	}
	
	fgets( buf, sizeof(buf) - 1, apmFp );
	buf[sizeof(buf) - 1] = '\0';
	fclose( apmFp );
	
	sscanf( buf, "%s %d.%d %x %x %x %x %d%%",
		useLess, /* driver version */
		&useLessToo, /* apm version major */
		&useLessToo, /* apm version minor */
		&useLessToo, /* apm flags */
		&recharging, /* ac line status */
		&useLessToo, /* battery_status */
		&useLessToo, /* battery_flags */
		&percentage /* battery_percentage */ );
	
	/* -1 means not present */
	if( percentage == -1 )
		percentage = 0;
	bat -> percentage = percentage;
	
	
	return recharging;
}


void APM_Update( )
{
	
	powerState.isCharging =
	powerState.batteryVector[0] -> filler( powerState.batteryVector[0] );
	powerState.percentage = powerState.batteryVector[0] -> percentage;
	
	setNewBatteryState( );
	return;
}



void setNewBatteryState( )
{
	static bool lastChargingState = false;
	static u_int8 lastPercentage = 0;
	
	if( lastChargingState != powerState.isCharging ||
	    lastPercentage != powerState.percentage )
	    draw_all();
	
	lastChargingState = powerState.isCharging;
	lastPercentage = powerState.percentage;
}



void estimatedTimeClock ( )
{
	/* we remember lst 20 percent values changes.
	 * for every value we record even the exact moment when this
	 * value was recorded (reading global variable secondsCounter) */
	static u_int32 values[5][2];
	static u_int8  valueIndex = ~0;
	
	static u_int16 lastEstimatedTime = ~0;
	
	register u_int8  diffPercentage = 0;
	register u_int32 diffTime       = 0;
	register int i;
	
	
	/* if battery charge is plugged in, values fetched before are
	 * reset to zero and return (draw_all() function already called
	 * by updater will draw TIME unlit, to indicate that battery isn't
	 * descharging. */
	if( powerState.isCharging )
	{
		valueIndex = ~0;
		return;
	}
	
	
	for( i = 4; i > 0; i-- )
	{
		values[i][1] = values[i-1][1];
		values[i][0] = values[i-1][0];
	}
	values[i][0] = powerState.percentage;
	values[i][1] = secondsCounter;
	
	
	valueIndex += 1;
	if( valueIndex > 4 )
		valueIndex = 4;
	
	/* if it's the first measure, jump the comparation */
	if( valueIndex > 0 )
	{
		u_int32 remainingSeconds;
		/* we have into values[0] the most recent value,
		 * and in values[ valueIndex ] the less one  */
		
		diffPercentage = values[ valueIndex ][0] - values[0][0];
		diffTime       = values[0][1] - values[ valueIndex ][1];
		
		/* this is the logic way: if diffTime is the number of seconds for
		 * losing diffPercentage percent battery charge, how many seconds will
		 * it takes to lose remaining points ? in other words:
		 * diffTime : diffPercentage = remainingSeconds : powerState.percentage
		 * it shouldn't be possible to have diffPercentage = 0
		 * because this function is called everytime that percent changes */
		remainingSeconds = ( diffTime * powerState.percentage ) / diffPercentage;
		
		/* value in minutes */
		powerState.remainingTime = remainingSeconds / 60;
	}
	else
		powerState.remainingTime = 0;
	
	
	if( powerState.remainingTime != lastEstimatedTime )
		draw_all();
	
	
	lastEstimatedTime = powerState.remainingTime;
	
#ifdef DEBUG
	printf("TEMPO TRASCORSO: %u:%02u   (%u secondi)\n",diffTime/60,diffTime%60,diffTime);
	printf("PERCENTUALI: values[0] = %u,  values[%d] = %u\n", values[0][0], valueIndex, values[valueIndex][0] );
	printf("DIFFENREZA: %d\n", diffPercentage );
	printf("TEMPOSTIMATO: %d\n\n\n", powerState.remainingTime );
#endif
}
