/*
 *  This program is free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation; either version 2 of the License, or
 *  (at your option) any later version.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with this program; if not, write to the Free Software
 *  Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
 */

#ifndef  __BATTERY_H__
#define  __BATTERY_H__

#include "main.h"



/*******************
 * ACPI FEATURES   *
 *********************************************************************************/
#define    ACPI_ACCESS_TEST      "/proc/acpi/info"

/* checks if it's possible to use ACPI support */
bool ACPI_canSupport( );


/* initializes data structures needed by program to run with ACPI support.
 * it returns SUCCESS if the initialization succeded, else ERROR. */
bool ACPI_init ( );

/* takes as argument the file path where information lies and it stores
 * into the second pointer the battery maximum capacity read from file.
 * it returns ERROR if it doesn't succeded in this operation. */
bool ACPI_maxCapacity ( char * , u_int32 *  );

/* is the function that periodically read battery data ( battery passed as
 * argument ), and updates its structure. */
bool ACPI_Filler ( battery );


/* if powerState.type is SUPPORT_ACPI, this function is called periodically 
 * to update battery values, percentage and battery charger. */
void ACPI_Update ( );


/***********************************************************************************
 * ACPI FEATURES   *
 *******************


 *******************
 * APM FEATURES    *
 ***********************************************************************************/

#define    APM_ACCESS_TEST      "/proc/apm"

/* checks if it's possible to use APM support */
bool APM_canSupport ( );

/* the same as ACPI, but for APM */
bool APM_init ( );


/* that's for ACPI comes in two distinct steps, in APM, battery status and
 * battery charger status (plugged in or not) is made the same file, so this
 * function fills 'percentage' field of battery structure passe as argument,
 * 'percentage' della struct batteria che gli viene passata, and returns TRUE
 * if battery charger is plugged in. */
bool APM_Filler ( battery );

/* the same for ACPI */
void APM_Update ( );

/*************************************************************************************
 * APM FEATURES    *
 *******************/


/* common function for ACPI and APM support that redraw the dockapp if battery
 * state is changed. */
void setNewBatteryState( );


/* function invoked every time that remaining battery percentage changes
 * and calculates estimated remaining time.
 * (if changed from previous updates the dockapp */
void estimatedTimeClock ( );


#endif
