/*
 *  This program is free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation; either version 2 of the License, or
 *  (at your option) any later version.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with this program; if not, write to the Free Software
 *  Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
 */

#include "event.h"



void event_handler ( )
{
	u_int32  update_seconds;
	u_int8   lastPercentage;
	int j;
	
	powerState.updater();
	update_seconds = time(NULL) - 2;
	lastPercentage = powerState.percentage;
	
	
	
	
	
	while( 1 )
	{
		secondsCounter = time(NULL);
		
		if( XPending( display )  )
		{
			XNextEvent( display, &e);
			
			switch( e.type)
			{
				case Expose:
					
					draw_all ( );
					
					break;
				
				case ButtonPress:
					
					j = CheckMouseRegion(e.xbutton.x, e.xbutton.y);
				
					switch ( j )
					{
						/* auto-freq */
						case MREGION_AUTOFREQ:
							/* toggle auto-freqency "(c)2003 LdP" scaling mode */
							cpuState.auto_freq_state = ( cpuState.auto_freq_state ? false: true );
							/* auto_freq_state is read from manageCpuLoad() function:
						   * if this is active, this function increase cpu frequency
							 * if it is at 100%, else decrease it. */
							break;
						/* < freq */
						case MREGION_MINFREQ:
							/* decrease cpu frequency */
							cpuSetFreq( FREQ_DECREMENT, FREQ_NEXT );
							cpuState.auto_freq_state = false;
							break;
						/* > freq */
						case MREGION_MAXFREQ:
							/* increase cpu frequency */
							cpuSetFreq( FREQ_INCREMENT, FREQ_NEXT );
							cpuState.auto_freq_state = false;
							break;
					}
					
					/* if a cliccable region was clicked, the dockapp is redrawn */
					if( j != -1 )
						draw_all ( );
						
					
					break;
			}
		}
		
		
		/* update battery percentage and cpuFreq
		 * a time every 2 seconds */
		if( update_seconds <= secondsCounter - 2 )
		{
			update_seconds = secondsCounter;
			powerState.updater();
			cpuCheckFreq();
		}
		
		
		/* estimated Time
		 * called every time that percent changes */
		if( lastPercentage != powerState.percentage )
		{
			lastPercentage = powerState.percentage;
			estimatedTimeClock ( );
		}
		
		/* check and redraw cpu load */
		manageCpuLoad();
		usleep( 100000 );
		
	}
}


void AddMouseRegion( u_int8 index, u_int8 left, u_int8 top, u_int8 right, u_int8 bottom)
{
	if (index < MAX_MOUSE_REGION) {
		mouse_region[index].top = top;
		mouse_region[index].left = left;
		mouse_region[index].bottom = bottom;
		mouse_region[index].right = right;
	}
}

u_int8 CheckMouseRegion( u_int8 x, u_int8 y)
{
	register int i;
	register bool found;
	
	found = 0;
	
	for (i=0; i<MAX_MOUSE_REGION && !found; i++) {
		if (x <= mouse_region[i].right &&
			x >= mouse_region[i].left &&
			y <= mouse_region[i].bottom &&
			y >= mouse_region[i].top)
			found = 1;
	}
	if (!found) return -1;
	return (i-1);
}
