/*
 *  This program is free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation; either version 2 of the License, or
 *  (at your option) any later version.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with this program; if not, write to the Free Software
 *  Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
 */

#include "init.h"


void init_display(  )
{
	XClassHint       classHint;
	XTextProperty    name;
	XGCValues        gcv;
	unsigned long    gcm;
	int              dummy=0;
	char           * progname = PROGNAME;
	
	Pixel            back_pix;
	Pixel            fore_pix;

	display = XOpenDisplay( args_XDisplayName );
	
	if( display == NULL )
	{
		fprintf(stderr, "Can't open display\n" );
		exit(1);
	}
	
	screen  = DefaultScreen(display);
	Root    = RootWindow(display, screen);
	d_depth = DefaultDepth(display, screen);
	x_fd    = XConnectionNumber(display);

	
	init_image ( );
	
	/* Create a window to hold the stuff */
	mysizehints.flags = USSize | USPosition;
	mysizehints.x = 0;
	mysizehints.y = 0;
	

	back_pix = WhitePixel( display, screen);
	fore_pix = BlackPixel( display, screen);

	XWMGeometry(display, screen, NULL, NULL, 1, &mysizehints, \
	            &mysizehints.x, &mysizehints.y,&mysizehints.width,&mysizehints.height, &dummy);
	
	mysizehints.width = 64;
	mysizehints.height = 64;
		
	win = XCreateSimpleWindow(display, Root, mysizehints.x, mysizehints.y, \
	                          mysizehints.width, mysizehints.height, 1, fore_pix, back_pix);
	
	iconwin = XCreateSimpleWindow(display, win, mysizehints.x, mysizehints.y, \
	                              mysizehints.width, mysizehints.height, 1, fore_pix, back_pix);

	/* Activate hints */
	XSetWMNormalHints(display, win, &mysizehints);
	classHint.res_name = progname;
	classHint.res_class = progname;
	XSetClassHint(display, win, &classHint);

	XSelectInput(display, win, ButtonPressMask | ExposureMask | StructureNotifyMask );
	XSelectInput(display, iconwin, ButtonPressMask | ExposureMask | StructureNotifyMask );

	if (XStringListToTextProperty(&progname, 1, &name) == 0)
	{
		fprintf(stderr, "%s: can't allocate window name\n", PROGNAME);
		exit(1);
	}

	XSetWMName(display, win, &name);
	
	/* Create GC for drawing */
	
	gcm = GCForeground | GCBackground | GCGraphicsExposures;
	gcv.foreground = fore_pix;
	gcv.background = back_pix;
	gcv.graphics_exposures = 0;
	NormalGC = XCreateGC(display, Root, gcm, &gcv);

	/* ONLYSHAPE ON */
	
	pixmask = XCreateBitmapFromData(display, win, wmlaptop_mask_bits, wmlaptop_mask_width, wmlaptop_mask_height);

	XShapeCombineMask(display, win, ShapeBounding, 0, 0, pixmask, ShapeSet);
	XShapeCombineMask(display, iconwin, ShapeBounding, 0, 0, pixmask, ShapeSet);
	
	/* ONLYSHAPE OFF */

	mywmhints.initial_state = WithdrawnState;
	mywmhints.icon_window = iconwin;
	mywmhints.icon_x = mysizehints.x;
	mywmhints.icon_y = mysizehints.y;
	mywmhints.window_group = win;
	mywmhints.flags = StateHint | IconWindowHint | IconPositionHint | WindowGroupHint;

	XSetWMHints(display, win, &mywmhints);
	
	XMapWindow(display, win);
	
	
}


void init_image ()
{
	char *xbm = wmlaptop_mask_bits;
	char ** xpm = XPM_NAME;
	int sx = wmlaptop_mask_width;
	int sy = wmlaptop_mask_height;
	
	XWindowAttributes      attributes;
	int                    err;
	
	int i,j,k;
	int width, height, numcol, depth;
    int zero=0;
    int bcount;
    int curpixel;
	unsigned char bwrite;
	
	sscanf(*xpm, "%d %d %d %d", &width, &height, &numcol, &depth);


    for (k=0; k!=depth; k++)
    {
        zero <<=8;
        zero |= xpm[1][k];
    }
        
	for (i=numcol+1; i < numcol+sy+1; i++)
	{
		bcount = 0;
		bwrite = 0;
		for (j=0; j<sx*depth; j+=depth)
		{
            bwrite >>= 1;

            curpixel=0;
            for (k=0; k!=depth; k++)
            {
                curpixel <<=8;
                curpixel |= xpm[i][j+k];
            }
                
            if ( curpixel != zero )
            {
				bwrite += 128;
			}
			bcount++;
			if (bcount == 8)
			{
				*xbm = bwrite;
				xbm++;
				bcount = 0;
				bwrite = 0;
			}
		}
	}
	
	/* For the colormap */
	XGetWindowAttributes(display, Root, &attributes);
	
	wmgen.attributes.valuemask |= (XpmReturnPixels | XpmReturnExtensions);
	
	err = XpmCreatePixmapFromData(display, Root, XPM_NAME, &(wmgen.pixmap), &(wmgen.mask), &(wmgen.attributes));
	
	if (err != XpmSuccess)
	{
		fprintf(stderr, "Not enough free colorcells.\n");
		exit(1);
	}
	
}

/* riempe la struttura cpuState tenendo anche conto
 * degli argomenti passati da linea di comando */
void init_cpuState ( )
{
	FILE *  fp;
	char red[10];
	char * paths[3][3] =
	          { /* sys     proc    user */
	/* min */   { "/sys/devices/system/cpu/cpu0/cpufreq/cpuinfo_min_freq",
	              "/proc/sys/cpu/0/speed-min",
	              args_useFileMin },
	/* max */   { "/sys/devices/system/cpu/cpu0/cpufreq/cpuinfo_max_freq",
	              "/proc/sys/cpu/0/speed-max",
	              args_useFileMax },
	/* set */   { "/sys/devices/system/cpu/cpu0/cpufreq/scaling_setspeed",
	              "/proc/sys/cpu/0/speed",
	              args_useFileSet }
	          };
	u_int8 idx[3] = {   /* the second index */
	     args_useFileMin ? 2 : ( args_useSysProc == USESYSPROC_SYS ? 0 : 1 ),
	     args_useFileMax ? 2 : ( args_useSysProc == USESYSPROC_SYS ? 0 : 1 ),
	     args_useFileSet ? 2 : ( args_useSysProc == USESYSPROC_SYS ? 0 : 1 )
	};
	
	
	/* auto_freq_state */
	cpuState.auto_freq_state = ( args_autoFreq == AUTOFREQ_ON );
	
	
	/* min freq */
	if( (fp = fopen( paths[0][idx[0]], "r" )) == NULL ) {
		fprintf( stderr, "Error reading the info file (%s):\n%s\n", paths[0][idx[0]], strerror(errno) );
		exit( ERROR );
	}
	
	fgets ( red, 10, fp );
	cpuState.minFreq = atoi ( red );
	
	fclose ( fp );
	
	/* max freq */	
	if( (fp = fopen( paths[1][idx[1]], "r" )) == NULL ) {
		fprintf( stderr, "Error reading the info file (%s):\n%s\n", paths[1][idx[1]], strerror(errno) );
		exit( ERROR );
	}
	
	fgets ( red, 10, fp );
	cpuState.maxFreq = atoi ( red );
	
	fclose ( fp );
	
	
	/* set freq file */
	cpuState.setFreqFile = paths[2][idx[2]];
	
	cpuReadFreq( );
	
	
	/* set freq */
	cpuState.setFreq = cpuState.actualFreq;
	
	/* step freq */
	cpuState.stepFreq = args_incrementalStep;
	
	
	/* altre opzioni */
	if( args_maxFreq != ARGSDEF_LEAVE ) {
		if( args_maxFreq > cpuState.maxFreq ) {
			fprintf( stderr, "Error: args_maxFreq > maxFreq possible (using %d as max)\n", cpuState.maxFreq );
			args_maxFreq = cpuState.maxFreq;
		}
		cpuState.maxFreq = args_maxFreq;
	}
	
	if( args_minFreq != ARGSDEF_LEAVE ) {
		if( args_minFreq < cpuState.minFreq ) {
			fprintf( stderr, "Error: args_minFreq < minFreq possible (using %d as min)\n", cpuState.minFreq );
			args_minFreq = cpuState.minFreq;
		}
		cpuState.minFreq = args_minFreq;
	}
	
	/* controllino.. */
	if( cpuState.minFreq > cpuState.maxFreq ) {
		fprintf( stderr, "Error: minFreq > maxFreq\n" );
		exit( ERROR );
	}
	
	switch( args_startingFreq )
	{
		case STARTINGFREQ_MIN:
			/* frequenza al minimo */
			while( cpuState.actualFreq != cpuState.minFreq )
				cpuSetFreq( FREQ_DECREMENT, FREQ_NEXT );
			break;
		case STARTINGFREQ_MAX:
			/* frequenza al massimo */
			while( cpuState.actualFreq != cpuState.maxFreq )
				cpuSetFreq( FREQ_INCREMENT, FREQ_NEXT );
	}
	
	
	
#ifdef DEBUG
	fprintf ( stderr, "min freq: %d\nmax freq: %d\nactual freq: %d\n", cpuState.minFreq, cpuState.maxFreq, cpuState.actualFreq );
#endif
	
	return;
}


void init_battery ( )
{
	if( args_useApmAcpi == USEAPMACPI_APM || args_useApmAcpi == USEAPMACPI_ONLYAPM )
	{
		if( APM_init() == ERROR )
		{
			if( args_useApmAcpi == USEAPMACPI_ONLYAPM )
				exit( ERROR );
			fprintf( stderr, "Switching to ACPI..\n");
			if( ACPI_init() == ERROR )
			{
				fprintf( stderr, ".. and now ?.. i must die\n" );
				exit( ERROR );
			}
		}
	}
	
	if( args_useApmAcpi == USEAPMACPI_ACPI || args_useApmAcpi == USEAPMACPI_ONLYACPI )
	{
		if( ACPI_init() == ERROR )
		{
			if( args_useApmAcpi == USEAPMACPI_ONLYACPI )
				exit( ERROR );
			fprintf( stderr, "Switching to APM..\n");
			if( APM_init() == ERROR )
			{
				fprintf( stderr, ".. and now ?.. i must die\n" );
				exit( ERROR );
			}
		}
	}
	
}
