/*
 *  This program is free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation; either version 2 of the License, or
 *  (at your option) any later version.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with this program; if not, write to the Free Software
 *  Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
 */
 
#include "main.h"


/* cpu's stuff */
struct cpuFreq cpuState;
       u_int8 cpuLoad;

/* battery's stuff */
struct power_str powerState;

/* time counter */
       time_t secondsCounter;

/* command line's args */
       char *  args_XDisplayName     = NULL;
       bool    args_autoFreq         = ARGSDEF_LEAVE;
       u_int32 args_incrementalStep  = ARGSDEF_LEAVE;
       bool    args_startingFreq     = ARGSDEF_LEAVE;
       u_int32 args_maxFreq          = ARGSDEF_LEAVE;
       u_int32 args_minFreq          = ARGSDEF_LEAVE;
       bool    args_useSysProc       = ARGSDEF_LEAVE;
       char *  args_useFileMax       = NULL;
       char *  args_useFileMin       = NULL;
       char *  args_useFileSet       = NULL;
       bool    args_useApmAcpi       = ARGSDEF_LEAVE;


/* X's stuff */
struct mouseRegion mouse_region[MAX_MOUSE_REGION];

       Display   * display;
       Window      Root;
       Window      iconwin;
       Window      win;
       int         screen;
       int         x_fd;
       int         d_depth;
       XSizeHints  mysizehints;
       XWMHints    mywmhints;
       GC          NormalGC;
       XEvent      e;
struct XpmIcon     wmgen;
       Pixmap      pixmask;

       char        wmlaptop_mask_bits[64 * 64];
       int         wmlaptop_mask_width = 64;
       int         wmlaptop_mask_height = 64;



int main( int argc, char * argv[] )
{
	
	setArgs( argc, argv );
	
	init_display  (  );
	
	init_battery  (  );
	
	init_cpuState (  );
	
	/* auto-freq mouse region */
	AddMouseRegion( MREGION_AUTOFREQ, 7, 36, 55, 46 );
	/* cpu frequency left arrow */
	AddMouseRegion( MREGION_MINFREQ, 4, 52, 7, 59 );
	/* cpu frequency right arrow */
	AddMouseRegion( MREGION_MAXFREQ, 55, 52, 58, 59 );
	
	event_handler (  );
	
	return SUCCESS;
}



/* prints help and exits */
void usage ( char * prog, char * unknowOpt )
{
	fprintf( stderr, "%s dockapp. An LdP-Production (version %s, build %d)\n",
	         PROGNAME, VERSION, BUILD_NR );
	fprintf( stderr, "It shows batteries' and cpu's state\n" );
	fprintf( stderr, "\n");
	fprintf( stderr, "[usage]: %s [options]\n", prog );
	fprintf( stderr, "\n");
	fprintf( stderr, "[X-options]:\n");
	fprintf( stderr, "   --display=NAME             set the display to open\n");
	fprintf( stderr, "[CPU-options]:\n");
	fprintf( stderr, "   --auto-freq=on|off         start with auto-scaling on|off\n");
	fprintf( stderr, "   --incremental-step=N       increment or decrement the cpu frequence by N\n");
	fprintf( stderr, "                              Khz per second when in auto-scaling mode\n");
	fprintf( stderr, "   --starting-freq=min|max    set the cpu frequence to the min|max possible\n");
	fprintf( stderr, "                              when start\n");
	fprintf( stderr, "   --max-freq=N               set manually the max and the min cpu frequence\n");
	fprintf( stderr, "   --min-freq=N               the dockapp must use as limit (in Khz)\n");
	fprintf( stderr, "   --use-sys                  use /sys/devices/system/cpu/cpu0/cpufreq/ to\n");
	fprintf( stderr, "                              control the cpu frequence scaling\n");
	fprintf( stderr, "   --use-proc                 use /proc/sys/cpu/0/ instead (OLD 2.4)\n");
	fprintf( stderr, "   --use-file-maxfreq=FILE    use FILE to read the maximum cpu frequence\n");
	fprintf( stderr, "   --use-file-minfreq=FILE    the minimum cpu frequence\n");
	fprintf( stderr, "   --use-file-setfreq=FILE    and to set the cpu frequence\n");
	fprintf( stderr, "[BATTERY-options]:\n");
	fprintf( stderr, "   --apm                      use APM kernel interface to monitoring battery\n");
	fprintf( stderr, "   --acpi                     use the modern ACPI interface instead\n");
	fprintf( stderr, "   --use-apm                  use APM, or quit if it fails\n");
	fprintf( stderr, "   --use-acpi                 use ACPI, or quit if it fails\n");
	fprintf( stderr, "[GENERAL-options]:\n");
	fprintf( stderr, "   -v   --version   show the version, and exit\n");
	fprintf( stderr, "   -h   --help      show this message, and exit\n");
	fprintf( stderr, "\n");
	
	if( unknowOpt ) {
		fprintf( stderr, "Unkown Options: %s\n", unknowOpt );
		fprintf( stderr, "\n");
		exit ( ERROR );
	}
	
	exit( SUCCESS );
}


void version ( )
{
	fprintf( stderr, "%s dockapp. An LdP-Production\n", PROGNAME );
	fprintf( stderr, "Version %s, build %d\n", VERSION, BUILD_NR );
	fprintf( stderr, "\n");
	fprintf( stderr, "Author: Giacomo, alias ]Matic[, Galilei:  matic at libero dot it\n");
	fprintf( stderr, "Author: Lorenzo, alias ]StClaus[, Marcon: stclaus at libero dot it\n");
	fprintf( stderr, "\n");
	exit( SUCCESS );
}




void setArgs( int argc, char ** argv )
{
	register int i;
	char *     ptr;
	
	for( i = 1; i < argc; i++ )
	{
		/* X Connection */
		if( !strncmp( argv[i], "--display=", 10 )) {
			EXIT_IF_ALREADY_SET( args_XDisplayName, NULL, "XDisplayName" );
			args_XDisplayName = strchr( argv[i], '=' );
			args_XDisplayName++;
			continue;
		}
		
		if( !strcmp( argv[i], "--auto-freq=on" )) {
			EXIT_IF_ALREADY_SET( args_autoFreq, ARGSDEF_LEAVE, "autoFreq" );
			args_autoFreq = AUTOFREQ_ON;
			continue;
		}
		
		if( !strcmp( argv[i], "--auto-freq=off" )) {
			EXIT_IF_ALREADY_SET( args_autoFreq, ARGSDEF_LEAVE, "autoFreq" );
			args_autoFreq = AUTOFREQ_OFF;
			continue;
		}
		
		if( !strncmp( argv[i], "--incremental-step=", 19 ) ) {
			EXIT_IF_ALREADY_SET( args_incrementalStep, ARGSDEF_LEAVE, "incrementalStep" );
			ptr = strchr( argv[i], '=' );
			ptr++;
			args_incrementalStep = atoi( ptr );
			WARNING_IS_SET_TO_ZERO( args_incrementalStep, "incrementalStep" );
			continue;
		}
		
		if( !strcmp( argv[i], "--starting-freq=min" ) ) {
			EXIT_IF_ALREADY_SET( args_startingFreq, ARGSDEF_LEAVE, "startingFreq" );
			args_startingFreq = STARTINGFREQ_MIN;
			continue;
		}
		
		if( !strcmp( argv[i], "--starting-freq=max" ) ) {
			EXIT_IF_ALREADY_SET( args_startingFreq, ARGSDEF_LEAVE, "startingFreq" );
			args_startingFreq = STARTINGFREQ_MAX;
			continue;
		}
		
		if( !strncmp( argv[i], "--max-freq=", 11 ) ) {
			EXIT_IF_ALREADY_SET( args_maxFreq, ARGSDEF_LEAVE, "maxFreq" );
			ptr = strchr( argv[i], '=' );
			ptr++;
			args_maxFreq = atoi( ptr );
			WARNING_IS_SET_TO_ZERO( args_maxFreq, "maxFreq" );
			continue;
		}
		
		if( !strncmp( argv[i], "--min-freq=", 11 ) ) {
			EXIT_IF_ALREADY_SET( args_minFreq, ARGSDEF_LEAVE, "minFreq" );
			ptr = strchr( argv[i], '=' );
			ptr++;
			args_minFreq = atoi( ptr );
			WARNING_IS_SET_TO_ZERO( args_minFreq, "minFreq" );
			continue;
		}
		
		if( !strcmp( argv[i], "--use-sys" )) {
			EXIT_IF_ALREADY_SET( args_useSysProc, ARGSDEF_LEAVE, "useSysProc" );
			args_useSysProc = USESYSPROC_SYS;
			continue;
		}
		
		if( !strcmp( argv[i], "--use-proc" )) {
			EXIT_IF_ALREADY_SET( args_useSysProc, ARGSDEF_LEAVE, "useSysProc" );
			args_useSysProc = USESYSPROC_PROC;
			continue;
		}
		
		if( !strncmp( argv[i], "--use-file-maxfreq=", 19 )) {
			EXIT_IF_ALREADY_SET( args_useFileMax, NULL, "useFileMax" );
			args_useFileMax = strchr( argv[i], '=' );
			args_useFileMax++;
			continue;
		}
		
		if( !strncmp( argv[i], "--use-file-minfreq=", 19 )) {
			EXIT_IF_ALREADY_SET( args_useFileMin, NULL, "useFileMin" );
			args_useFileMin = strchr( argv[i], '=' );
			args_useFileMin++;
			continue;
		}
		
		if( !strncmp( argv[i], "--use-file-setfreq=", 19 )) {
			EXIT_IF_ALREADY_SET( args_useFileSet, NULL, "useFileSet" );
			args_useFileSet = strchr( argv[i], '=' );
			args_useFileSet++;
			continue;
		}
		
		if( !strcmp( argv[i], "--apm" )) {
			EXIT_IF_ALREADY_SET( args_useApmAcpi, ARGSDEF_LEAVE, "useApmAcpi" );
			args_useApmAcpi = USEAPMACPI_APM;
			continue;
		}
		
		if( !strcmp( argv[i], "--acpi" )) {
			EXIT_IF_ALREADY_SET( args_useApmAcpi, ARGSDEF_LEAVE, "useApmAcpi" );
			args_useApmAcpi = USEAPMACPI_ACPI;
			continue;
		}
		
		if( !strcmp( argv[i], "--use-apm" )) {
			EXIT_IF_ALREADY_SET( args_useApmAcpi, ARGSDEF_LEAVE, "useApmAcpi" );
			args_useApmAcpi = USEAPMACPI_ONLYAPM;
			continue;
		}
		
		if( !strcmp( argv[i], "--use-acpi" )) {
			EXIT_IF_ALREADY_SET( args_useApmAcpi, ARGSDEF_LEAVE, "useApmAcpi" );
			args_useApmAcpi = USEAPMACPI_ONLYACPI;
			continue;
		}
		
		if( !strcmp( argv[i], "-v" ) || !strcmp( argv[i], "--version" ))
			version( );
		
		if( !strcmp( argv[i], "-h" ) || !strcmp( argv[i], "--help" ))
			usage( argv[0], NULL );
		
		usage( argv[0], argv[i] );
	}
	
	/* setting default values not set by command line */
	SET_DEFAULT( args_XDisplayName,    NULL,          ARGSDEF_XDISPLAYNAME );
	SET_DEFAULT( args_autoFreq,        ARGSDEF_LEAVE, ARGSDEF_AUTOFREQ );
	SET_DEFAULT( args_incrementalStep, ARGSDEF_LEAVE, ARGSDEF_INCREMENTALSTEP );
	SET_DEFAULT( args_startingFreq,    ARGSDEF_LEAVE, ARGSDEF_STARTINGFREQ );
	SET_DEFAULT( args_maxFreq,         ARGSDEF_LEAVE, ARGSDEF_MAXFREQ );
	SET_DEFAULT( args_minFreq,         ARGSDEF_LEAVE, ARGSDEF_MINFREQ );
	SET_DEFAULT( args_useSysProc,      ARGSDEF_LEAVE, ARGSDEF_USESYSPROC );
	SET_DEFAULT( args_useFileMax,      NULL,          ARGSDEF_USEFILEMAX );
	SET_DEFAULT( args_useFileMin,      NULL,          ARGSDEF_USEFILEMIN );
	SET_DEFAULT( args_useFileSet,      NULL,          ARGSDEF_USEFILESET );
	SET_DEFAULT( args_useApmAcpi,      ARGSDEF_LEAVE, ARGSDEF_USEAPMACPI );
	
}
